@@ -67,14 +67,6 @@ check_header (GsfInputGZip *input)
 	guint8 const *data;
 	unsigned flags, len;
 
-	/* Get the uncompressed size first, so that the seeking is finished */
-	if (gsf_input_seek (input->source, (gsf_off_t) -4, G_SEEK_END) ||
-	    NULL == (data = gsf_input_read (input->source, 4, NULL)))
-		return TRUE;
-	/* FIXME, but how?  The size read here is modulo 2^32.  */
-	gsf_input_set_size (GSF_INPUT (input),
-			    (gsf_off_t) GSF_LE_GET_GUINT32 (data));
-
 	/* Check signature */
 	if (gsf_input_seek (input->source, (gsf_off_t) 0, G_SEEK_SET) ||
 	    NULL == (data = gsf_input_read (input->source, 2 + 1 + 1 + 6, NULL)) ||
@@ -86,6 +78,18 @@ check_header (GsfInputGZip *input)
 	if (data [2] != Z_DEFLATED || (flags & ~GZIP_HEADER_FLAGS) != 0)
 		return TRUE;
 
+	/* Get the uncompressed size */
+	if (gsf_input_seek (input->source, (gsf_off_t) -4, G_SEEK_END) ||
+	    NULL == (data = gsf_input_read (input->source, 4, NULL)))
+		return TRUE;
+	/* FIXME, but how?  The size read here is modulo 2^32.  */
+	gsf_input_set_size (GSF_INPUT (input),
+			    (gsf_off_t) GSF_LE_GET_GUINT32 (data));
+
+	if (gsf_input_seek (input->source, (gsf_off_t) 2 + 1 + 1 + 6,
+			    G_SEEK_SET))
+		return TRUE;
+
 	if (flags & GZIP_EXTRA_FIELD) {
 		if (NULL == (data = gsf_input_read (input->source, 2, NULL)))
 			return TRUE;
@@ -115,6 +119,36 @@ check_header (GsfInputGZip *input)
 
 	input->header_size = input->source->cur_offset;
 
+	if (gsf_input_remaining (input->source) < 9)
+		return TRUE;	/* No room for payload */
+
+	return FALSE;
+}
+
+static gboolean
+init_zip (GsfInputGZip *gzip, GError **err)
+{
+	gsf_off_t cur_pos;
+	
+	if (Z_OK != inflateInit2 (&(gzip->stream), -MAX_WBITS)) {
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error (), 0,
+				"Unable to initialize zlib");
+		return TRUE;
+	}
+
+	cur_pos = gsf_input_tell (gzip->source);
+
+	if (check_header (gzip) != FALSE) {
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error (), 0,
+				"Invalid gzip header");
+		if (gsf_input_seek (gzip->source, cur_pos, G_SEEK_SET)) {
+			g_warning ("attempt to restore position failed ??");
+		}
+		return TRUE;
+	}
+
 	return FALSE;
 }
 
@@ -131,6 +165,7 @@ GsfInputGZip *
 gsf_input_gzip_new (GsfInput *source, GError **err)
 {
 	GsfInputGZip *gzip;
+	gsf_off_t  cur_pos;
 
 	g_return_val_if_fail (GSF_IS_INPUT (source), NULL);
 
@@ -139,17 +174,7 @@ gsf_input_gzip_new (GsfInput *source, GError **err)
 	gzip->source = source;
 	gzip->seek_skipped = 0;
 
-	if (Z_OK != inflateInit2 (&(gzip->stream), -MAX_WBITS)) {
-		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
-				"Unable to initialize zlib");
-		g_object_unref (G_OBJECT (gzip));
-		return NULL;
-	}
-	if (check_header (gzip)) {
-		if (err != NULL)
-			*err = g_error_new (gsf_input_error (), 0,
-				"Invalid gzip header");
+	if (init_zip (gzip, err) != FALSE) {
 		g_object_unref (G_OBJECT (gzip));
 		return NULL;
 	}
@@ -184,10 +209,12 @@ gsf_input_gzip_dup (GsfInput *src_input, GError **err)
 	GsfInputGZip const *src = (GsfInputGZip *)src_input;
 	GsfInputGZip *dst = g_object_new (GSF_INPUT_GZIP_TYPE, NULL);
 
-	(void) err;
+	dst->source = gsf_input_dup(src->source, NULL);
 
-	dst->source = src->source;
-	g_object_ref (G_OBJECT (dst->source));
+	if (init_zip (dst, err) != FALSE) {
+		g_object_unref (G_OBJECT (dst));
+		return NULL;
+	}
 
 	return GSF_INPUT (dst);
 }
