@@ -25,20 +25,21 @@
 #include <pygobject.h>
 
 extern PyMethodDef pygsf_functions[];
+extern DL_EXPORT(void) init_gsf (void);
 
 DL_EXPORT(void)
-initgsf (void)
+init_gsf (void)
 {
 	PyObject *m, *d;
 
 	init_pygobject ();
 
-	m = Py_InitModule ("gsf", pygsf_functions);
+	m = Py_InitModule ((char *) "_gsf", pygsf_functions);
 	d = PyModule_GetDict (m);
 
 	pygsf_register_classes (d);
 	
 	if (PyErr_Occurred ()) {
-		Py_FatalError ("can't initialise module gsf");
+		Py_FatalError ((char *) "can't initialise module gsf");
 	}
 }
